"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FeelInput = void 0;
var jsx_runtime_1 = require("react/jsx-runtime");
var Monaco = require("@kie-tools-core/monaco-editor");
var React = require("react");
var react_1 = require("react");
var FeelConfigs_1 = require("./FeelConfigs");
Monaco.languages.register({
    id: FeelConfigs_1.MONACO_FEEL_LANGUAGE,
    aliases: [FeelConfigs_1.MONACO_FEEL_LANGUAGE, "feel", "feel-dmn"],
    mimetypes: ["text/feel"],
});
Monaco.editor.defineTheme(FeelConfigs_1.MONACO_FEEL_THEME, (0, FeelConfigs_1.feelTheme)());
Monaco.languages.setMonarchTokensProvider(FeelConfigs_1.MONACO_FEEL_LANGUAGE, (0, FeelConfigs_1.feelTokensConfig)());
var __firstTimeInitializingMonacoToEnableColorizingCorrectly = true;
exports.FeelInput = React.forwardRef(function (_a, forwardRef) {
    var enabled = _a.enabled, value = _a.value, suggestionProvider = _a.suggestionProvider, onBlur = _a.onBlur, onPreviewChanged = _a.onPreviewChanged, onKeyDown = _a.onKeyDown, onChange = _a.onChange, options = _a.options;
    var monacoContainer = (0, react_1.useRef)(null);
    var monacoRef = (0, react_1.useRef)();
    (0, react_1.useEffect)(function () {
        if (!__firstTimeInitializingMonacoToEnableColorizingCorrectly) {
            return;
        }
        console.info("Registered FEEL language on Monaco Editor to enable correct 'colorize' call.");
        Monaco.editor
            .create(monacoContainer.current, { theme: FeelConfigs_1.MONACO_FEEL_THEME, language: FeelConfigs_1.MONACO_FEEL_LANGUAGE })
            .dispose();
        __firstTimeInitializingMonacoToEnableColorizingCorrectly = false;
    }, []);
    (0, react_1.useEffect)(function () {
        if (!enabled) {
            return;
        }
        var disposable = Monaco.languages.registerCompletionItemProvider(FeelConfigs_1.MONACO_FEEL_LANGUAGE, {
            provideCompletionItems: function (model, position) {
                var completionItems = (0, FeelConfigs_1.feelDefaultSuggestions)();
                if (suggestionProvider) {
                    var items = suggestionProvider(model.getValue(), position.lineNumber, position.column - 1);
                    if (items.length > 0) {
                        completionItems = items;
                    }
                }
                return {
                    suggestions: completionItems,
                };
            },
        });
        return function () {
            disposable.dispose();
        };
    }, [enabled, suggestionProvider]);
    var config = (0, react_1.useMemo)(function () {
        return (0, FeelConfigs_1.feelDefaultConfig)(options);
    }, [options]);
    (0, react_1.useEffect)(function () {
        if (enabled && !monacoRef.current) {
            var element = monacoContainer.current;
            var editor_1 = Monaco.editor.create(element, config);
            editor_1.onDidChangeModelContent(function (event) {
                var value = editor_1.getValue();
                Monaco.editor.colorize(value, FeelConfigs_1.MONACO_FEEL_LANGUAGE, {}).then(function (colorizedValue) {
                    onChange === null || onChange === void 0 ? void 0 : onChange(event, value, colorizedValue);
                });
            });
            editor_1.onDidBlurEditorWidget(function () {
                onBlur === null || onBlur === void 0 ? void 0 : onBlur(editor_1.getValue());
            });
            editor_1.onKeyDown(function (e) {
                onKeyDown === null || onKeyDown === void 0 ? void 0 : onKeyDown(e, editor_1.getValue());
            });
            editor_1.focus();
            monacoRef.current = editor_1;
        }
    }, [config, enabled, onBlur, onChange, onKeyDown]);
    (0, react_1.useEffect)(function () {
        var _a, _b;
        if (enabled) {
            (_a = monacoRef.current) === null || _a === void 0 ? void 0 : _a.setValue(value !== null && value !== void 0 ? value : "");
            (_b = monacoRef.current) === null || _b === void 0 ? void 0 : _b.setPosition(calculatePosition(value !== null && value !== void 0 ? value : ""));
        }
    }, [value, enabled]);
    (0, react_1.useEffect)(function () {
        var _a, _b;
        if (!enabled && monacoRef.current) {
            onBlur === null || onBlur === void 0 ? void 0 : onBlur((_a = monacoRef.current) === null || _a === void 0 ? void 0 : _a.getValue());
            (_b = monacoRef.current) === null || _b === void 0 ? void 0 : _b.dispose();
            monacoRef.current = undefined;
        }
    }, [enabled, onBlur]);
    (0, react_1.useEffect)(function () {
        Monaco.editor.colorize(value !== null && value !== void 0 ? value : "", FeelConfigs_1.MONACO_FEEL_LANGUAGE, {}).then(function (colorizedValue) {
            onPreviewChanged === null || onPreviewChanged === void 0 ? void 0 : onPreviewChanged(colorizedValue);
        });
    }, [onPreviewChanged, value]);
    (0, react_1.useImperativeHandle)(forwardRef, function () { return ({
        setMonacoValue: function (newValue) { var _a; return (_a = monacoRef.current) === null || _a === void 0 ? void 0 : _a.setValue(newValue); },
        getMonacoValue: function () { var _a; return (_a = monacoRef.current) === null || _a === void 0 ? void 0 : _a.getValue(); },
        insertNewLineToMonaco: function () { var _a; return (_a = monacoRef.current) === null || _a === void 0 ? void 0 : _a.trigger("keyboard", "type", { text: "\n" }); },
        isSuggestionWidgetOpen: function () { var _a, _b; return (_b = (_a = monacoRef.current) === null || _a === void 0 ? void 0 : _a._contentWidgets["editor.widget.suggestWidget"]) === null || _b === void 0 ? void 0 : _b.position; },
    }); });
    return ((0, jsx_runtime_1.jsx)("div", __assign({ className: "feel-input" }, { children: (0, jsx_runtime_1.jsx)("div", { ref: monacoContainer }) })));
});
function calculatePosition(value) {
    var lines = value.split("\n");
    var lineNumber = lines.length;
    var column = lines[lineNumber - 1].length + 1;
    return { lineNumber: lineNumber, column: column };
}
//# sourceMappingURL=FeelInput.js.map